#include <wx/wx.h>
#include <wx/file.h>
#include <wx/filename.h>
#include <wx/xrc/xmlres.h>

#include "cryptutil.h"

#include "appres.h"
#include "xh_appres.h"
#include "app.h"
#include "frame.h"


void InitXmlResource();


IMPLEMENT_APP(MyApp)

bool MyApp::OnInit()
{
    wxXmlResource *res = wxXmlResource::Get();
    res->InitAllHandlers();
    res->AddHandler(new AppResHandler);
    InitXmlResource();
    
    _res = wxDynamicCast(res->LoadObject(NULL, _T("appres"), _T("AppRes")),
                         AppRes);
    
    MyFrame *frame = new MyFrame();
    frame->Show(TRUE);
    SetTopWindow(frame);
    
    if (argc > 1) {
        wxString filename(argv[1]);
        if (wxFileName::FileExists(filename)) {
            frame->LoadUI(filename);
        }
    }

    return TRUE;
} 


int MyApp::OnExit()
{
    delete _res;
    cleanup_openssl_library();
    return 0;
}


const AppRes* MyApp::GetAppRes() const
{
    return _res;
}


bool MyApp::SaveFile(const wxString &filename, const wxString &data,
                     const wxString &password)
{
    bool ret = FALSE;
    crypthelper_t *helper = crypthelper_alloc();
    if (helper) {
        wxCharBuffer bdata = data.utf8_str();
        wxCharBuffer bpw = password.utf8_str();
        if (cryptutil_encrypt(helper, bdata, strlen(bdata), bpw, strlen(bpw)) == 0) {
            wxFile f(filename, wxFile::write);
            f.Write(crypthelper_getptr(helper), crypthelper_getlen(helper));
            f.Close();
            ret = TRUE;
        }
        crypthelper_free(helper);
    }
    return ret;
}


bool MyApp::LoadFile(const wxString &filename, wxString *data,
                     const wxString &password)
{
    filename, data, password;
    bool ret = FALSE;
    crypthelper_t *helper = crypthelper_alloc();
    if (helper) {
        wxCharBuffer bpw(password.utf8_str());
        wxFile f(filename, wxFile::read);
        size_t blen = f.Length();
        char *bdata = new char[blen];
        f.Read(bdata, blen);
        f.Close();
        if (cryptutil_decrypt(helper, bdata, blen, bpw, strlen(bpw)) == 0) {
            *data = wxString::FromUTF8(crypthelper_getptr(helper),
                                       crypthelper_getlen(helper));
            ret = TRUE;
        }
        delete[] bdata;
        crypthelper_free(helper);
    }
    return ret;
}

