#include <string.h>

#include <wx/wx.h>
#include <wx/datetime.h>
#include <wx/file.h>
#include <wx/textctrl.h>
#include <wx/xrc/xmlres.h>
#include <wx/fdrepdlg.h>
#include <wx/fontdlg.h>
#include <wx/printdlg.h>
#include <wx/config.h>

#include "appres.h"
#include "app.h"
#include "dialog.h"
#include "frame.h"


BEGIN_EVENT_TABLE(MyFrame, wxFrame)
    
    EVT_MENU(XRCID("mi_new"), MyFrame::OnNew)
    EVT_MENU(XRCID("mi_open"), MyFrame::OnOpen)
    EVT_MENU(XRCID("mi_save"), MyFrame::OnSave)
    EVT_MENU(XRCID("mi_save_as"), MyFrame::OnSaveAs)
    EVT_MENU(XRCID("mi_quit"), MyFrame::OnQuit)
    
    EVT_MENU(XRCID("mi_undo"), MyFrame::OnUndo)
    EVT_MENU(XRCID("mi_cut"), MyFrame::OnCut)
    EVT_MENU(XRCID("mi_copy"), MyFrame::OnCopy)
    EVT_MENU(XRCID("mi_paste"), MyFrame::OnPaste)
    EVT_MENU(XRCID("mi_delete"), MyFrame::OnDelete)
    EVT_MENU(XRCID("mi_find"), MyFrame::OnFind)
    EVT_MENU(XRCID("mi_find_next"), MyFrame::OnFindNext)
    EVT_MENU(XRCID("mi_replace"), MyFrame::OnReplace)
    EVT_MENU(XRCID("mi_jump"), MyFrame::OnJump)
    EVT_MENU(XRCID("mi_select_all"), MyFrame::OnSelectAll)
    EVT_MENU(XRCID("mi_insert_date"), MyFrame::OnInsertDate)
    
    EVT_MENU(XRCID("mi_wrap"), MyFrame::OnWrap)
    EVT_MENU(XRCID("mi_font"), MyFrame::OnFont)
    
    EVT_MENU(XRCID("mi_statusbar"), MyFrame::OnStatusBar)
    
    EVT_MENU(XRCID("mi_about"), MyFrame::OnAbout)
    
    EVT_UPDATE_UI(XRCID("mi_undo"), MyFrame::OnUpdateUndo)
    EVT_UPDATE_UI(XRCID("mi_cut"), MyFrame::OnUpdateCut)
    EVT_UPDATE_UI(XRCID("mi_copy"), MyFrame::OnUpdateCopy)
    EVT_UPDATE_UI(XRCID("mi_paste"), MyFrame::OnUpdatePaste)
    EVT_UPDATE_UI(XRCID("mi_delete"), MyFrame::OnUpdateDelete)
    EVT_UPDATE_UI(XRCID("mi_find"), MyFrame::OnUpdateFind)
    EVT_UPDATE_UI(XRCID("mi_find_next"), MyFrame::OnUpdateFindNext)
    
    EVT_UPDATE_UI(XRCID("mi_wrap"), MyFrame::OnUpdateWrap)
    EVT_UPDATE_UI(XRCID("mi_statusbar"), MyFrame::OnUpdateStatusBar)
    
    
    EVT_FIND(wxID_ANY, MyFrame::OnFindFind)
    EVT_FIND_NEXT(wxID_ANY, MyFrame::OnFindFind)
    EVT_FIND_REPLACE(wxID_ANY, MyFrame::OnFindReplace)
    EVT_FIND_REPLACE_ALL(wxID_ANY, MyFrame::OnFindReplaceAll)
    EVT_FIND_CLOSE(wxID_ANY, MyFrame::OnFindClose)
    
    
    EVT_CLOSE(MyFrame::OnClose)
    
    EVT_TEXT(wxID_ANY, MyFrame::OnText)
    
    EVT_DROP_FILES(MyFrame::OnDropFiles)
    
END_EVENT_TABLE()


BEGIN_EVENT_TABLE(TextCtrlWatcher, wxEvtHandler)
    EVT_KEY_DOWN(TextCtrlWatcher::OnKey)
    EVT_KEY_UP(TextCtrlWatcher::OnKey)
    EVT_LEFT_DOWN(TextCtrlWatcher::OnMouse)
    EVT_LEFT_UP(TextCtrlWatcher::OnMouse)
    EVT_LEFT_DCLICK(TextCtrlWatcher::OnMouse)
END_EVENT_TABLE()


static const wxChar* str_find_back(const wxChar *s, const wxChar *find, long end)
{
    const wxChar *q = s + end;
    const wxChar *cur = s;
    const wxChar *found;
    const wxChar *found0 = NULL;
    int flen = wcslen(find);
    for (; ; ) {
        found = wcsstr(cur, find);
        if (!found || found + flen > q) {
            break;
        }
        found0 = found;
        cur = found + 1;
    }
    return found0;
}


static long StringFindForward(const wxString &s, const wxString &find,
    long start, bool sense_case=true)
{
    wxString s0(s);
    wxString find0(find);
    if (!sense_case) {
        s0 = s.Lower();
        find0 = find.Lower();
    }
    long found = s0.Mid(start).Find(find0);
    return found != wxNOT_FOUND ? start + found : wxNOT_FOUND;
}


static long StringFindBackward(const wxString &s, const wxString &find,
    long end, bool sense_case=true)
{
    wxString s0(s);
    wxString find0(find);
    if (!sense_case) {
        s0 = s.Lower();
        find0 = find.Lower();
    }
    const wxChar *s_ = s0.c_str();
    const wxChar *find_ = find0.c_str();
    const wxChar *found_ = str_find_back(s_, find_, end);
    return found_ ? found_ - s_ : wxNOT_FOUND;
}


MyFrame::MyFrame()
    :wxFrame(NULL, wxID_ANY, _T("")),
    _filename(),
    _password(),
    _datetime_format(),
    _statusbar_format(),
    _textctrl(NULL),
    _finddata(NULL),
    _finddlg(NULL),
    _statusbar(false)
{
    DragAcceptFiles(TRUE);
    
    wxMenuBar *menuBar = wxXmlResource::Get()->LoadMenuBar(_T("menubar"));
    SetMenuBar(menuBar);
#ifdef __WXMSW__
    wxIcon icon(_T("SEALEDPAD_APP"), wxBITMAP_TYPE_ICO_RESOURCE);
    SetIcon(icon);
#endif
    
    const AppRes *res = ::wxGetApp().GetAppRes();
    _datetime_format = res->GetString(_T("fmt_datetime"));
    _statusbar_format = res->GetString(_T("fmt_statusbar"));
    
    _textctrl = new wxTextCtrl(this, wxID_ANY, wxEmptyString,
        wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_NOHIDESEL);
    _textctrl->PushEventHandler(new TextCtrlWatcher(this));
    
    _finddata = new wxFindReplaceData(wxFR_DOWN);
    
    LoadConfig();
    SetFileName(_T(""), _T(""));
}


MyFrame::~MyFrame()
{
    delete _finddata;
    if (_textctrl) _textctrl->PopEventHandler(TRUE);
}


bool MyFrame::Show(bool show)
{
    bool ret = wxFrame::Show(show);
    
    if (show) _textctrl->SetSize(GetClientSize());
    
    return ret;
}


void MyFrame::SaveConfig()
{
    wxConfigBase *cfg = wxConfigBase::Get();
    if (!cfg) return;
    
    bool maximized = IsMaximized();
    cfg->Write(_T("WindowMaximized"), maximized);
    
    if (!IsMaximized() && !IsIconized()) {
        wxRect rwin = GetScreenRect();
        cfg->Write(_T("WindowPosX"), rwin.x);
        cfg->Write(_T("WindowPosY"), rwin.y);
        cfg->Write(_T("WindowPosWidth"), rwin.width);
        cfg->Write(_T("WindowPosHeight"), rwin.height);
    }
    
    wxFont font = _textctrl->GetFont();
    cfg->Write(_T("FontInfo"), font.GetNativeFontInfoDesc());
    cfg->Write(_T("Wrap"), GetWrap());
    cfg->Write(_T("StatusBar"), _statusbar);

    wxConfigBase::Set(NULL);
    delete cfg;
}


void MyFrame::LoadConfig()
{
    wxConfigBase *cfg = wxConfigBase::Get();
    if (!cfg) return;

    bool maximized = FALSE;
    if (cfg->Read(_T("WindowMaximized"), &maximized)) {
        Maximize(maximized);
    }
    if (!maximized) {
        wxRect rwin;
        if (cfg->Read(_T("WindowPosX"), &rwin.x)
            && cfg->Read(_T("WindowPosY"), &rwin.y)
            && cfg->Read(_T("WindowPosWidth"), &rwin.width)
            && cfg->Read(_T("WindowPosHeight"), &rwin.height))
        {
            SetSize(rwin);
        }
    }
    
    wxString fontinfo;
    if (cfg->Read(_T("FontInfo"), &fontinfo)) {
        wxFont font;
        font.SetNativeFontInfo(fontinfo);
        if (font.IsOk()) _textctrl->SetFont(font);
    }
    
    bool wrap;
    if (cfg->Read(_T("Wrap"), &wrap)) {
        SetWrap(wrap);
    }
    
    bool showsb;
    if (cfg->Read(_T("StatusBar"), &showsb)) {
        ShowStatusBar(showsb);
    }
    
    wxConfigBase::Set(NULL);
    delete cfg;
}


void MyFrame::ShowStatusBar(bool show)
{
    _statusbar = show;
    wxStatusBar *bar = GetStatusBar();
    
    if (GetWrap()) show = false;
    
    if (!!show == !!bar) return;
    
    if (show) {
        wxStatusBar *bar = CreateStatusBar();
        int widths[] = {-3, -1};
        bar->SetFieldsCount(2, widths);
        SetStatusBar(bar);
        UpdateStatusBar();
    } else {
        SetStatusBar(NULL);
        delete bar;
    }
}


wxString MyFrame::GetFileName() const
{
    return _filename;
}


bool MyFrame::GetWrap() const
{
    return !(_textctrl->GetWindowStyle() & wxHSCROLL);
}


void MyFrame::SetWrap(bool wrap)
{
    if (!wrap == !GetWrap()) return;
    
    wxTextCtrl *old_textctrl = _textctrl;
    
    wxTextCtrl *new_textctrl = new wxTextCtrl(this, wxID_ANY,
        old_textctrl->GetValue(), wxDefaultPosition, wxDefaultSize,
        old_textctrl->GetWindowStyle()^wxHSCROLL);
    
    new_textctrl->SetFont(old_textctrl->GetFont());
    new_textctrl->SetModified(old_textctrl->IsModified());
    
    old_textctrl->PopEventHandler(TRUE);
    delete old_textctrl;
    
    _textctrl = new_textctrl;
    
    _textctrl->PushEventHandler(new TextCtrlWatcher(this));
    _textctrl->SetSize(GetClientSize());
    _textctrl->Refresh();
    _textctrl->SetFocus();
    
    ShowStatusBar(_statusbar);
}


void MyFrame::SetFileName(const wxString &filename, const wxString &password)
{
    const AppRes *res = ::wxGetApp().GetAppRes();
    
    _filename = filename;
    _password = password;
    
    wxString name = wxFileName(filename).GetFullName();
    if (!name) name = res->GetString(_T("untitled"));
    
    wxString fmt = res->GetString(_T("fmt_frame"));
    wxString appname = res->GetString(_T("appname"));
    wxString title = wxString::Format(fmt.c_str(),
                                      name.c_str(), appname.c_str());
    SetTitle(title);
}


bool MyFrame::Delete()
{
    long from;
    long to;
    _textctrl->GetSelection(&from, &to);
    if (from != to) {
        _textctrl->Remove(from, to);
        return true;
    }
    return false;
}


void MyFrame::FindUI(bool replace)
{
    if (_finddlg) {
        _finddlg->SetFocus();
        return;
    }
    
    const AppRes *res = ::wxGetApp().GetAppRes();
    _finddlg = new wxFindReplaceDialog(this, _finddata,
        res->GetString(replace ? _T("replace") : _T("find")),
        (replace ? wxFR_REPLACEDIALOG : 0)|wxFR_NOWHOLEWORD);
    _finddlg->Show();
}


bool MyFrame::FindForward(const wxString& find)
{
    long pos;
    long from, to;
    _textctrl->GetSelection(&from, &to);
    pos = from > to ? from : to;
    long col, row;
    _textctrl->PositionToXY(pos, &col, &row);
    long maxrow = _textctrl->GetNumberOfLines();
    long found;
    wxString s;
    bool sense_case = !!(_finddata->GetFlags() & wxFR_MATCHCASE);
    for (; row < maxrow; row++) {
        s = _textctrl->GetLineText(row);
        found = StringFindForward(s, find, col, sense_case);
        if (found != wxNOT_FOUND) {
            from = _textctrl->XYToPosition(found, row);
            to = from + find.Length();
            _textctrl->SetSelection(from, to);
            return TRUE;
        }
        col = 0;
    }
    return FALSE;
}


bool MyFrame::FindBackward(const wxString& find)
{
    long pos;
    long from, to;
    _textctrl->GetSelection(&from, &to);
    pos = from < to ? from : to;
    long col, row;
    _textctrl->PositionToXY(pos, &col, &row);
    long found;
    wxString s;
    bool sense_case = !!(_finddata->GetFlags() & wxFR_MATCHCASE);
    for (; row >= 0; row--) {
        s = _textctrl->GetLineText(row);
        if (col == -1) col = s.Length();
        found = ::StringFindBackward(s, find, col, sense_case);
        if (found != wxNOT_FOUND) {
            from = _textctrl->XYToPosition(found, row);
            to = from + find.Length();
            _textctrl->SetSelection(from, to);
            return TRUE;
        }
        col = -1;
    }
    return FALSE;
}


bool MyFrame::FindNext()
{
    bool found;
    if (_finddata->GetFlags() & wxFR_DOWN) {
        found = FindForward(_finddata->GetFindString());
    } else {
        found = FindBackward(_finddata->GetFindString());
    }
    if (!found) {
        const AppRes *res = ::wxGetApp().GetAppRes();
        wxString title = res->GetString(_T("appname"));
        wxString fmt = res->GetString(_T("fmt_not_found"));
        wxString find = _finddata->GetFindString();
        ::wxMessageBox(wxString::Format(fmt.c_str(), find.c_str()), title,
            wxOK|wxICON_INFORMATION);
    }
    return found;
}


void MyFrame::FindNextUI()
{
    if (_finddlg || _finddata->GetFindString().IsEmpty()) {
        FindUI();
    } else {
        FindNext();
    }
}



void MyFrame::ReplaceAll(const wxString& find, const wxString& replace)
{
    wxString data = _textctrl->GetValue();
    if (data.Replace(find, replace, TRUE)) {
        _textctrl->ChangeValue(data);
        _textctrl->MarkDirty();
        _textctrl->SetInsertionPoint(0);
    }
}


void MyFrame::UpdateStatusBar()
{
    wxStatusBar *bar = GetStatusBar();
    if (!bar) return;
    
    long pos = _textctrl->GetInsertionPoint();
    long row, col;
    _textctrl->PositionToXY(pos, &col, &row);
    bar->SetStatusText(wxString::Format(_statusbar_format, row+1, col+1), 1);
}


bool MyFrame::Load(const wxString& filename, const wxString& password)
{
    wxString data;
    if (MyApp::LoadFile(filename, &data, password)) {
        _textctrl->ChangeValue(data);
        SetFileName(filename, password);
        _textctrl->SetFocus();
        return TRUE;
    }
    return FALSE;
}


bool MyFrame::LoadUI(const wxString& filename)
{
    const AppRes *res = ::wxGetApp().GetAppRes();
    PasswordDialog2 dlg_password(filename);
    wxRect r = GetScreenRect();
    wxSize s = dlg_password.GetSize();
    dlg_password.Move(r.x+r.width/2-s.GetWidth()/2,
        r.y+r.height/2-s.GetHeight()/2);
    while (dlg_password.ShowModal() == wxID_OK) {
        if (Load(filename, dlg_password.GetValue()))
            return TRUE;
        ::wxMessageBox(res->GetString(_T("open_error")),
            res->GetString(_T("open")), wxOK|wxICON_EXCLAMATION);
        dlg_password.InitFocus();
        return TRUE;
    }
    return FALSE;
}


void MyFrame::New()
{
    _textctrl->Clear();
    _textctrl->DiscardEdits();
    
    _finddata->SetFlags(wxFR_DOWN);
    _finddata->SetFindString(_T(""));
    _finddata->SetReplaceString(_T(""));
    
    SetFileName(_T(""), _T(""));
    UpdateStatusBar();
}


bool MyFrame::OpenUI()
{
    const AppRes *res = ::wxGetApp().GetAppRes();
    wxFileDialog dlg_open(this,
        res->GetString(_T("open")),
        _T(""), _T(""),
        res->GetString(_T("wildcard")),
        wxFD_OPEN|wxFD_FILE_MUST_EXIST);
    
    while (dlg_open.ShowModal() == wxID_OK) {
        if (LoadUI(dlg_open.GetPath())) break;
    }
    
    return FALSE;
}


bool MyFrame::SaveAs(const wxString& filename, const wxString& password)
{
    if (MyApp::SaveFile(filename, _textctrl->GetValue(), password)) {
        _textctrl->DiscardEdits();
        SetFileName(filename, password);
        return TRUE;
    }
    return FALSE;
}


bool MyFrame::Save()
{
    if (_filename) return SaveAs(_filename, _password);
    return FALSE;
}


bool MyFrame::SaveAsUI()
{
    const AppRes *res = ::wxGetApp().GetAppRes();
    
    wxFileName fn(_filename);
    
    wxFileDialog dlg_saveas(this, res->GetString(_T("save_as")),
        fn.GetPath(), fn.GetFullName(), res->GetString(_T("wildcard")),
        wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    
    while (dlg_saveas.ShowModal() == wxID_OK) {
        for (; ; ) {
            PasswordDialog1 dlg_password(dlg_saveas.GetPath());
            wxRect r = GetScreenRect();
            wxSize s = dlg_password.GetSize();
            dlg_password.Move(r.x+r.width/2-s.GetWidth()/2,
                r.y+r.height/2-s.GetHeight()/2);
            if (dlg_password.ShowModal() != wxID_OK) break;
            if (dlg_password.IsConfirmed()) {
                
                if (SaveAs(dlg_saveas.GetPath(), dlg_password.GetValue()))
                    return TRUE;
                
                ::wxMessageBox(res->GetString(_T("save_error")),
                    res->GetString(_T("save_as")), wxOK|wxICON_EXCLAMATION);
                
                return FALSE;
            }
            ::wxMessageBox(res->GetString(_T("confirm_error")),
                res->GetString(_T("save_as")), wxOK|wxICON_EXCLAMATION);
        }
    }
    return FALSE;
}


bool MyFrame::SaveUI()
{
    if (_filename.IsEmpty())
        return SaveAsUI();
    
    if (Save())
        return TRUE;
    
    const AppRes *res = ::wxGetApp().GetAppRes();
    ::wxMessageBox(res->GetString(_T("save_error")),
        res->GetString(_T("save")), wxOK|wxICON_EXCLAMATION);
    
    return FALSE;
}


bool MyFrame::AskSaveUI()
{
    if (_textctrl->IsModified()) {
        const AppRes *res = ::wxGetApp().GetAppRes();
        wxString fmt = res->GetString(_T("fmt_ask_save"));
        wxString name = _filename.IsEmpty() ? res->GetString(_T("untitled"))
                                            : _filename;
        wxString msg = wxString::Format(fmt.c_str(), name.c_str());
        wxString appname = res->GetString(_T("appname"));
        int ans = ::wxMessageBox(msg, appname,
                                 wxICON_EXCLAMATION|wxYES_NO|wxCANCEL);
        switch (ans) {
        case wxYES:
            return SaveUI();
        case wxNO:
            return TRUE;
        case wxCANCEL:
            return FALSE;
        default:
            break;
        }
    }
    return TRUE;
}


void MyFrame::OnClose(wxCloseEvent& event)
{
    if (!AskSaveUI()) return;
    
    SaveConfig();
    event.Skip();
}


void MyFrame::OnNew(wxCommandEvent& WXUNUSED(event))
{
    if (AskSaveUI()) New();
}


void MyFrame::OnOpen(wxCommandEvent& WXUNUSED(event))
{
    if (AskSaveUI()) OpenUI();
}


void MyFrame::OnSave(wxCommandEvent& WXUNUSED(event))
{
    SaveUI();
}


void MyFrame::OnSaveAs(wxCommandEvent& WXUNUSED(event))
{
    SaveAsUI();
}

void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
    Close();
}


void MyFrame::OnUndo(wxCommandEvent& WXUNUSED(event))
{
    _textctrl->Undo();
}


void MyFrame::OnCut(wxCommandEvent& WXUNUSED(event))
{
    _textctrl->Cut();
}


void MyFrame::OnCopy(wxCommandEvent& WXUNUSED(event))
{
    _textctrl->Copy();
}


void MyFrame::OnPaste(wxCommandEvent& WXUNUSED(event))
{
    _textctrl->Paste();
}


void MyFrame::OnDelete(wxCommandEvent& WXUNUSED(event))
{
    Delete();
}


void MyFrame::OnFind(wxCommandEvent& WXUNUSED(event))
{
    FindUI();
}


void MyFrame::OnFindNext(wxCommandEvent& WXUNUSED(event))
{
    FindNextUI();
}


void MyFrame::OnReplace(wxCommandEvent& WXUNUSED(event))
{
    FindUI(TRUE);
}


void MyFrame::OnJump(wxCommandEvent& WXUNUSED(event))
{
    long pos = _textctrl->GetInsertionPoint();
    long row, col;
    _textctrl->PositionToXY(pos, &col, &row);
    
    JumpDialog dlg_jump;
    dlg_jump.Move(_textctrl->GetScreenPosition());
    dlg_jump.SetValue(row+1);
    if (dlg_jump.ShowModal() != wxID_OK) return;
    
    pos = _textctrl->XYToPosition(0, dlg_jump.GetValue()-1);
    _textctrl->SetInsertionPoint(pos);
}


void MyFrame::OnSelectAll(wxCommandEvent& WXUNUSED(event))
{
    _textctrl->SetSelection(-1, -1);
}


void MyFrame::OnInsertDate(wxCommandEvent& WXUNUSED(event))
{
    _textctrl->WriteText(wxDateTime::Now().Format(_datetime_format));
}


void MyFrame::OnWrap(wxCommandEvent& WXUNUSED(event))
{
    SetWrap(!GetWrap());
}


void MyFrame::OnFont(wxCommandEvent& WXUNUSED(event))
{
    wxFont origfont = _textctrl->GetFont();
    wxFont newfont = ::wxGetFontFromUser(this, origfont);
    if (newfont.IsOk()) {
        _textctrl->SetFont(newfont);
    }
}


void MyFrame::OnStatusBar(wxCommandEvent& WXUNUSED(event))
{
    ShowStatusBar(!_statusbar);
}


void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
    AboutDialog dlg_about;
    wxRect r = GetScreenRect();
    wxSize s = dlg_about.GetSize();
    dlg_about.Move(r.x+r.width/2-s.GetWidth()/2,
        r.y+r.height/2-s.GetHeight()/2);
    dlg_about.ShowModal();
}


void MyFrame::OnUpdateUndo(wxUpdateUIEvent& event)
{
    event.Enable(_textctrl->CanUndo());
}


void MyFrame::OnUpdateCut(wxUpdateUIEvent& event)
{
    event.Enable(_textctrl->CanCut());
}


void MyFrame::OnUpdateCopy(wxUpdateUIEvent& event)
{
    event.Enable(_textctrl->CanCopy());
}


void MyFrame::OnUpdatePaste(wxUpdateUIEvent& event)
{
    event.Enable(_textctrl->CanPaste());
}


void MyFrame::OnUpdateDelete(wxUpdateUIEvent& event)
{
    long from;
    long to;
    _textctrl->GetSelection(&from, &to);
    event.Enable(from != to);
}


void MyFrame::OnUpdateFind(wxUpdateUIEvent& event)
{
    event.Enable(!_textctrl->IsEmpty());
}


void MyFrame::OnUpdateFindNext(wxUpdateUIEvent& event)
{
    event.Enable(!_textctrl->IsEmpty());
}


void MyFrame::OnUpdateWrap(wxUpdateUIEvent& event)
{
    event.Check(GetWrap());
}


void MyFrame::OnUpdateStatusBar(wxUpdateUIEvent& event)
{
    event.Check(!!GetStatusBar());
    event.Enable(!GetWrap());
}


void MyFrame::OnFindFind(wxFindDialogEvent& WXUNUSED(event))
{
    FindNext();
}


void MyFrame::OnFindReplace(wxFindDialogEvent& WXUNUSED(event))
{
    long from, to;
    _textctrl->GetSelection(&from, &to);
    if (_textctrl->GetRange(from, to) == _finddata->GetFindString()) {
        _textctrl->Replace(from, to, _finddata->GetReplaceString());
    }
    FindNext();
}


void MyFrame::OnFindReplaceAll(wxFindDialogEvent& WXUNUSED(event))
{
    const wxString find = _finddata->GetFindString();
    if (find) {
        ReplaceAll(find, _finddata->GetReplaceString());
    }
}


void MyFrame::OnFindClose(wxFindDialogEvent& WXUNUSED(event))
{
    _finddlg = NULL;
}


void MyFrame::OnDropFiles(wxDropFilesEvent& event)
{
    wxString *pfilename = event.GetFiles();
    if (AskSaveUI()) LoadUI(*pfilename);
}


void MyFrame::OnText(wxCommandEvent& WXUNUSED(event))
{
    UpdateStatusBar();
}


TextCtrlWatcher::TextCtrlWatcher(MyFrame *parent)
{
    _parent = parent;
}


void TextCtrlWatcher::OnKey(wxKeyEvent &event)
{
    _parent->UpdateStatusBar();
    event.Skip();
}


void TextCtrlWatcher::OnMouse(wxMouseEvent &event)
{
    _parent->UpdateStatusBar();
    event.Skip();
}
